import java.util.Arrays;
import java.util.Random;

/**
   This program measures the runtime of the bubble
   sort algorithm on a sequence of growing arrays.

   For an array of length n, the runtime of bubble
   sort is approximately n^2. Bubble sort is a
   "quadratic" algorithm.
       T(n) ~ n^2
   This is sometimes written as O(n^2).

   Here is an important consequence of this.
   Approximate the runtime of an array that
   has been doubled in size, from n to 2n.

       T(2n) ~ (2n)^2 = 2^2 * n^2 = 4n^2 = 4T(n)

   When you double the size of an array, it takes
   quadruple the amount of time to sort it. The
   data collected by this program shows that this
   calculation is correct.
*/
public class BubbleSort_Profile
{
   public static void main(String[] args)
   {
      final int size = 10_000;   // starting size for the arrays
      final int iterations = 12; // how many times to increase the array size

      System.out.println("Bubble sort.");

      for (int i = 1; i <= iterations; ++i)
      {
         final int[] array = BubbleSort.randomIntArray(i*size, 1_000_000);

         final long startTime = System.currentTimeMillis();
            BubbleSort.sort(array);
         final long stopTime = System.currentTimeMillis();

         System.out.println("For n = " + (i*size) + ", time = " + (stopTime - startTime));
      }
   }
}
