import java.util.Arrays;
import java.util.Random;

/**
   This program measures the runtime of the merge
   sort algorithm on a sequence of growing arrays.

   For each size n of the array, we sort three kinds
   of arrays, an already sorted array, a random array,
   and an array that is sorted backwards. The first array,
   since it is already sorted, should be the best case. The
   second array, since it is random, should be the average
   case. And the third array, since it is the opposite of
   being sorted, should be the worst case.
*/
public class MergeSort_Profile_v2
{
   public static void main(String[] args)
   {
      final int size = 1_000_000; // starting size for the arrays
      final int iterations = 20;  // how many times to increase the array size

      System.out.println("Merge sort.");

      for (int i = 1; i <= iterations; ++i)
      {
         int[] array = new int[i*size];
         for (int j = 0; j < array.length; ++j)
         {
            array[j] = j;
         }
         long startTime = System.currentTimeMillis();
            MergeSort.sort(array);
         long stopTime = System.currentTimeMillis();
         final long bestCase =(stopTime - startTime);

         array = MergeSort.randomIntArray(i*size, 1_000_000);
         startTime = System.currentTimeMillis();
            MergeSort.sort(array);
         stopTime = System.currentTimeMillis();
         final long avgCase =(stopTime - startTime);

         array = new int[i*size];
         for (int j = 0; j < array.length; ++j)
         {
            array[j] = i*size - j;
         }
         startTime = System.currentTimeMillis();
            MergeSort.sort(array);
         stopTime = System.currentTimeMillis();
         final long worstCase =(stopTime - startTime);

         System.out.println(
            "For n = " + (i*size) + ", "
          + "best time = " + bestCase + ", "
          + "avg time = " + avgCase + ", "
          + "worst time = " + worstCase);
      }
   }
}
