import java.util.Random;
import java.util.Arrays;

/**
   Define and demonstrate the quicksort
   algorithm on integer arrays.
*/
public class QuickSort
{
   /**
      This main() method initializes an array with random values
      and then calls the quicksort algorithm on the array.
   */
   public static void main(String[] args)
   {
      final int[] a = randomIntArray(32, 100);
      System.out.println( Arrays.toString(a) );
      sort(a);
      System.out.println( Arrays.toString(a) );
      System.out.println();

      for (int j = 0; j < a.length; ++j)
      {
         a[j] = j;
      }
      System.out.println( Arrays.toString(a) );
      sort(a);
      System.out.println( Arrays.toString(a) );
      System.out.println();

      for (int j = 0; j < a.length; ++j)
      {
         a[j] = a.length - j;
      }
      System.out.println( Arrays.toString(a) );
      sort(a);
      System.out.println( Arrays.toString(a) );
   }


   /**
      Create an array filled with random values.

      @param length  the length of the returned array
      @param n       the number of possible random values
      @return an array filled with length numbers
              between 0 and n - 1
   */
   public static int[] randomIntArray(int length, int n)
   {
      final Random generator = new Random();

      final int[] a = new int[length];
      for (int i = 0; i < a.length; ++i)
      {
         a[i] = generator.nextInt(n);
      }
      return a;
   }



   /**
      Sort an array using quicksort.

      @param a  the array to sort
   */
   public static void sort(int[] a)
   {
      sort(a, 0, a.length - 1); // sort the whole array
   }


   /**
      Sort a portion of an array using quicksort.

      @param a   the array to sort
      @param lo  the first index of the portion to be sorted
      @param hi  the last index of the portion to be sorted
   */
   public static void sort(int[] a, int lo, int hi)
   {
      if (lo >= hi) // base case
      {
         return;
      }
      final int p = partition(a, lo, hi); // Divide.
//      System.out.println("left = " + (p-lo+1) + ", right = " + (hi-p));
      sort(a, lo,  p);                    // Conquer left part.
      sort(a, p+1, hi);                   // Conquer right part.
   }                                      // No combine step.


   /**
      Partition a portion of an array.

      @param a   the array to partition
      @param lo  the first index of the portion to be partitioned
      @param hi  the last index of the portion to be partitioned
      @return the last index of the first partition
   */
   private static int partition(int[] a, int lo, int hi)
   {
      final int pivot = a[lo];
      int i = lo - 1;
      int j = hi + 1;
      while (i < j)
      {
         ++i; while (a[i] < pivot) { ++i; }
         --j; while (a[j] > pivot) { --j; }
         if (i < j)
         {
            swap(a, i, j);
         }
      }
      return j;
   }


   /**
      Swap two entries of an array.

      @param a  the array
      @param i  the first position to swap
      @param j  the second position to swap
   */
   private static void swap(int[] a, int i, int j)
   {
      final int temp = a[i];
      a[i] = a[j];
      a[j] = temp;
   }
}
