import java.util.Arrays;
import java.util.Random;

/**
   Define and demonstrate an iterative selection
   sort algorithm on integer arrays.
*/
public class SelectionSort_Loop
{
   /**
      This main() method initializes an array with random values
      and then calls the insertion sort algorithm on the array.
   */
   public static void main(String[] args)
   {
      final int[] a = randomIntArray(25, 100);
      System.out.println( Arrays.toString(a) );

      sort(a);
      System.out.println( Arrays.toString(a) );
   }


   /**
      Create an array filled with random values.

      @param length  the length of the returned array
      @param n       the number of possible random values
      @return an array filled with length numbers
              between 0 and n - 1
   */
   public static int[] randomIntArray(int length, int n)
   {
      final Random generator = new Random();

      final int[] a = new int[length];
      for (int i = 0; i < a.length; ++i)
      {
         a[i] = generator.nextInt(n);
      }
      return a;
   }



   /**
      Sort an array using iterative selection sort.

      @param a  the array to sort
   */
   public static void sort(int[] a)
   {
      for (int front = 0; front < a.length - 1; ++front)
      {
         final int minPos = minimumPosition(a, front);
         swap(a, minPos, front);
      }
   }


   /**
      Find the smallest element in a tail range of the array.

      @param a          the array to sort
      @param fromIndex  the first position in a to compare
      @return the position of the smallest element in the
              range a[fromIndex] . . . a[a.length - 1]
   */
   private static int minimumPosition(int[] a, int fromIndex)
   {
      int minPosition = fromIndex; // initial guess of where the min is positioned
      for (int i = fromIndex + 1; i < a.length; ++i)
      {
         if (a[i] < a[minPosition])
         {
            minPosition = i; // update the guess
         }
      }
      return minPosition;
   }


   /**
      Swap two entries of an array.

      @param a  the array
      @param i  the first position to swap
      @param j  the second position to swap
   */
   public static void swap(int[] a, int i, int j)
   {
      final int temp = a[i];
      a[i] = a[j];
      a[j] = temp;
   }
}
